@@ -22,8 +22,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSRunLoop class reference</title>
    $Date$ $Revision$
@@ -426,6 +425,12 @@ static inline BOOL timerInvalidated(NSTimer *t)
 #if HAVE_DISPATCH_MAIN_QUEUE_DRAIN_NP
   dispatch_main_queue_drain_np();
 #elif HAVE__DISPATCH_MAIN_QUEUE_CALLBACK_4CF
+#if defined(__linux__)
+  uint64_t value;
+  int fd = (int)(intptr_t)data;
+  int n = eventfd_read(fd, &value);
+  (void) n;
+#endif
   _dispatch_main_queue_callback_4CF(NULL);
 #else
 #error libdispatch missing main queue callback function
@@ -780,6 +785,9 @@ static inline BOOL timerInvalidated(NSTimer *t)
           NSInvocation		        *inv;
           NSTimer                       *timer;
           SEL			        sel;
+          #ifdef RL_INTEGRATE_DISPATCH
+          static GSMainQueueDrainer 	*drainer = nil;
+          #endif
 
           ctr = [NSNotificationCenter defaultCenter];
           not = [NSNotification notificationWithName: @"GSHousekeeping"
@@ -802,17 +810,20 @@ static inline BOOL timerInvalidated(NSTimer *t)
           [current addTimer: timer forMode: NSDefaultRunLoopMode];
 
           #ifdef RL_INTEGRATE_DISPATCH
-          // We leak the queue drainer, because it's integral part of RL
-          // operations
-          GSMainQueueDrainer *drain =
-            [NSObject leak: [[GSMainQueueDrainer new] autorelease]];
+	  if (nil == drainer)
+	    {
+	      /* We leak the queue drainer, because it's integral part of RL
+	       * operations
+	       */
+	      drainer = [GSMainQueueDrainer new];
+	    }
           [current addEvent: [GSMainQueueDrainer mainQueueFileDescriptor]
 #ifdef _WIN32
                        type: ET_HANDLE
 #else
                        type: ET_RDESC
 #endif
-                    watcher: drain
+                    watcher: drainer
                     forMode: NSDefaultRunLoopMode];
 
           #endif
