@@ -17,8 +17,7 @@
 
    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110 USA.
+   Software Foundation, Inc., 31 Milk Street #960789 Boston, MA 02196 USA.
 
    <title>NSUndoManager class reference</title>
 */
@@ -429,6 +428,11 @@
  */
 - (void) forwardInvocation: (NSInvocation*)anInvocation
 {
+  NSUInteger    size = [[anInvocation methodSignature] methodReturnLength];
+  unsigned char v[size];
+
+  memset(v, '\0', size);
+
   if (_disableCount == 0)
     {
       if (_nextTarget == nil)
@@ -466,6 +470,8 @@
 	  _runLoopGroupingPending = YES;
 	}
     }
+
+  [anInvocation setReturnValue: (void*)v];
 }
 
 /**
